/*
  RobotFace.h - Arduino Library for controlling RobotFace (MiRAE).
  Created by Casey Bennett, May 2013.
  www.CaseyBennett.com

   Copyright 2013 Casey Bennett

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License (Lesser GPL) as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RobotFace_h
#define RobotFace_h

#include "Arduino.h"
#include "MyTypes.h"
#include "Servo.h"

class RobotFace
{
  public:
    //Servo Globals 
    Servo Lt_Eye;
    Servo Rt_Eye;
    Servo Lt_Brow;
    Servo Rt_Brow;
    Servo Lt_MC;
    Servo Rt_MC;
    Servo Lt_BrowRaise;
    Servo Rt_BrowRaise;
    Servo Lt_MouthPart;
    Servo Rt_MouthPart;
    Servo Neck_Tilt;
    Servo Neck_Pan;
    
    //Constructor
    RobotFace(int lt_eye_init, int rt_eye_init, int lt_brow_init, int rt_brow_init, int lt_mc_init, int rt_mc_init, int lt_br_init,
                  int rt_br_init, int lt_mp_init, int rt_mp_init, int nt_init, int np_init,
                  int x_center, int x_min, int x_max, int y_center, int y_min, int y_max, 
                  int x_move, int y_move);
    
    //Useful Functions 
    void Initialize();
    void Set_Degree(double et_degree=1.0, double bt_degree=1.0, double mc_degree=1.0, double br_degree=1.0, double mp_degree=1.0, 
                           double nt_degree=1.0, double np_degree=1.0);   
    Returns Get_Curr_Pos();    
    void Neck_Move(double delta_nt, double delta_np);
    void Robot_Look(long x, long y, int fixed_flag);
    void Look_Center();
    void Emerg_Center();
    void Eye_Saccade();
    
    //Facial Expression Functions
    void Sad_Face(int dir, int neck_on);                                 
    void Happy_Face(int dir, int neck_on);                                 
    void Angry_Face(int dir, int neck_on);
    void Fear_Face(int dir, int neck_on);
    void Surprise_Face(int dir, int neck_on);
    void Interest_Face(int dir, int neck_on);
    void Make_Face(int face_choice, int delay_time, int neck_on);
    
    //Communication Functions
    long getSerial();    
    
  private:
    //Initial Facial Component Positions
    int lt_eye_init;                    //Left Eye Tilt
    int rt_eye_init;                    //Right Eye Tilt
    int lt_brow_init;                   //Left Brow Raise
    int rt_brow_init;                   //Right Brow Raise
    int lt_mc_init;                     //Left Mouth Corner
    int rt_mc_init;                     //Right Mouth Corner
    int lt_br_init;                     //Left Brow Raise
    int rt_br_init;                     //Right Brow Raise
    int lt_mp_init;                     //Left Mouth Part
    int rt_mp_init;                     //Right Mouth Part
    int nt_init;                        //Neck Tilt
    int np_init;                        //Neck Pan
    
    //Facial Component Motion Degrees
    double et_deg;
    double bt_deg;
    double mc_deg;
    double br_deg;
    double mp_deg;
    double nt_deg;
    double np_deg;
    
    //Current Positions
    int curr_et_l;                      // eye tilt - left
    int curr_et_r;                      // eye tilt - right
    int curr_bt_l;                      // brow tilt - left
    int curr_bt_r;                      // brow tilt - right
    int curr_mc_l;                      // mouth corner - left
    int curr_mc_r;                      // mouth corner - right
    int curr_br_l;                      // brow raise - left
    int curr_br_r;                      // brow raise - right
    int curr_mp_l;                      // mouth part - left (open/close lips)
    int curr_mp_r;                      // mouth part - right (open/close lips)
    int curr_nt;                        // neck tilt
    int curr_np;                        // neck pan
        
    //Camera Globals
    int x_center;
    int x_min;
    int x_max;
    int y_center;
    int y_min;
    int y_max;    
    //Move Globals
    int x_move;                          //Max motion in x direction for any single movement
    int y_move;                          //Max motion in y direction for any single movement
    //Move Constraints
    int nt_min;
    int nt_max;
    int np_min;
    int np_max;
};

#endif
